require 'spec_helper'

ID = 23
ACTIVE = false
CREATED_AT = "01/01/01 23:23:59"
UPDATED_AT = "02/02/02 23:23:59"
EMAIL = 'test@testing.com'
AGE = 6

class TestModel
  include RD3::Model

  attribute :email
  attribute :age

  validates :email, :presence => true
  validates :age, :presence => true
end

describe RD3::Model do

  let(:test_model) { TestModel.new(:id => ID, :email => EMAIL,
                                   :age => AGE, :active => ACTIVE,
                                   :created_at => CREATED_AT, :updated_at => UPDATED_AT) }

  context 'initializing' do

    it 'should initialize without attributes hash' do
      tm = TestModel.new
      tm.should_not be_nil
      tm.active.should be_true # default value

      # dirty should be false when calling parameterless 'new' method
      tm.dirty?.should be_false

      # set the id
      tm.id = 5

      # number of class attributes (array)
      tm.class_attributes.count.should eq(6)

      # number of attributes (hash of only those properties that have been set thus far)
      # so, in this example, both active and id properties have been set
      tm.attributes.count.should eq(2)
    end

    it 'should initialize with attributes hash' do
      test_model.should_not be_nil
      test_model.email.should eq(EMAIL)
      test_model.age.should eq(AGE)
      test_model.active.should eq(ACTIVE)

      # dirty should be true when initializing with a hash
      test_model.dirty?.should be_true
    end

    it 'should initialize with class instance' do
      tm = TestModel.new(test_model)
      tm.should_not be_nil
      tm.id.should eq(test_model.id)
      tm.email.should eq(test_model.email)
      tm.age.should eq(test_model.age)
      tm.active.should eq(test_model.active)
      tm.created_at.should eq(test_model.created_at)
      tm.updated_at.should eq(test_model.updated_at)

      # dirty should be false when calling 'new' method with an instance of a class
      tm.dirty?.should be_false
    end

  end # context 'initializing'

  it 'should set dirty flag' do
    tm = TestModel.new
    tm.should_not be_nil
    tm.dirty?.should be_false

    tm.age = 6
    tm.dirty?.should be_true
  end

  it 'should not be persisted' do
    TestModel.new.persisted?.should be_false
  end

  it 'should be persisted' do
    test_model.persisted?.should be_true
  end

  it 'should have class attributes' do
    test_model.class_attributes.should_not be_nil
    test_model.class_attributes.is_a?(Array)
    test_model.class_attributes.include?(:id).should be_true
    test_model.class_attributes.include?(:active).should be_true
    test_model.class_attributes.include?(:created_at).should be_true
    test_model.class_attributes.include?(:updated_at).should be_true
    test_model.class_attributes.include?(:email).should be_true
    test_model.class_attributes.include?(:age).should be_true

    # dirty should not be a class attribute!!
    test_model.class_attributes.include?(:dirty).should be_false
    test_model.class_attributes.count.should eq(6)
  end

  it 'should have instance attributes' do
    test_model.attributes.should_not be_nil
    test_model.attributes.count.should eq(6)
    test_model.attributes.is_a?(Hash)
    test_model.attributes[:id].should eq(ID)
    test_model.attributes[:active].should eq(ACTIVE)
    test_model.attributes[:created_at].should eq(CREATED_AT)
    test_model.attributes[:updated_at].should eq(UPDATED_AT)
    test_model.attributes[:email].should eq(EMAIL)
    test_model.attributes[:age].should eq(AGE)
    test_model.attributes[:dirty].should be_nil
  end

  it 'should raise an exception when getting a value for a missing property' do
    expect { test_model.duodenum }.to raise_error(NoMethodError, /undefined method/)
  end

  it 'should raise an exception when setting a value for a missing property' do
    expect { test_model.duodenum = 6 }.to raise_error(NoMethodError, /undefined method/)
  end

  it 'should transfer errors' do
    test_model.email = nil
    test_model.age = nil
    test_model.valid?.should be_false
    test_model.errors.count.should eq(2)

    test_model2 =  TestModel.new(:email => 'testing@rspec.org', :age => 16)
    test_model2.valid?.should be_true
    test_model2.errors.count.should eq(0)
    test_model2.errors = test_model.errors
    test_model2.errors.count.should eq(2)
  end

  it 'should call to_json' do
    json = test_model.to_json
    json.class.should eq(String)
    json.length.should eq(138)
  end

  it 'should call as_json' do
    json = test_model.as_json
    json.class.should eq(Hash)
    json.length.should eq(7)
    json['id'].should eq(ID)
    json['active'].should eq(ACTIVE)
    json['created_at'].should eq(CREATED_AT)
    json['updated_at'].should eq(UPDATED_AT)
    json['email'].should eq(EMAIL)
    json['age'].should eq(AGE)
    json['dirty'].should be_true
  end

end
