require 'securerandom'

shared_examples "data provider" do |config_file_name|
  let(:is_document_based) {(/Rdbms/ =~ repo.to_s).nil?}

  it 'should have config file name' do
    repo.config_file_name.should eq(config_file_name)
  end

  it 'should have properly configured client' do
    repo.client.should_not be_nil
  end

  context 'persisting and querying' do

    it 'should persist new instance and then find by key' do
      _create_new_instance_and_verify
    end

    it 'should not find by invalid key' do
      repo.find_by_key(-1).should be_nil
    end

    it 'should not find by key if active flag is false and exclude logical deletes is true' do
      instance = _create_new_instance_and_verify(false)
      id = _get_db_instance_id(instance)
      repo.find_by_key(id, _build_find_by_opts(instance, true)).should be_nil
    end

    it 'should find by key if active flag is false and exclude logical deletes is false' do
      instance = _create_new_instance_and_verify(false)
      id = _get_db_instance_id(instance)
      repo.find_by_key(id, _build_find_by_opts(instance, false)).should_not be_nil
    end

    it 'should update existing instance' do
      instance = _create_new_instance_and_verify
      id = _get_db_instance_id(instance)
      instance[:email] = "updated_#{instance[:email]}"
      instance[:age] = 82
      instance[:amt] = 8.88
      instance[:active] = false

      repo.save(instance)

      updated_instance = repo.find_by_key(id, _build_find_by_opts(instance, false))
      updated_instance.should_not be_nil
      updated_instance[:email].should eq(instance[:email])
      updated_instance[:age].should eq(instance[:age])
      updated_instance[:amt].to_f.should eq(instance[:amt])
      updated_instance[:active].should eq(instance[:active])
    end

    it 'should perform config based logical delete' do
      RD3.config.enable_logical_deletes = true
      _create_new_instance_and_delete(nil)
    end

    it 'should perform config based physical delete' do
      RD3.config.enable_logical_deletes = false
      _create_new_instance_and_delete(nil)
    end

    it 'should perform explicit logical delete' do
      _create_new_instance_and_delete(true)
    end

    it 'should perform explicit physical delete' do
      _create_new_instance_and_delete(false)
    end

    def _create_new_instance_and_verify(active=true)
      # attributes that will be perisisted regardless of
      # data provider
      email = "spec#{Random.rand(100)}@spec_testing.com"
      age = Random.rand(60)
      amt = Random.rand(1..9) + 0.99
      attributes = {:email => email, :age => age, :amt => amt, :active => active}

      # if we're dealing w/ a document based data provider,
      # then generate a document key
      if is_document_based
        attributes.merge!(:key => SecureRandom.uuid)
        # row key is required by azure table storage
        # doesn't hurt if it's persisted in a different document store
        # such as mongo or couchbase
        attributes.merge!(:row_key => Time.now.to_i.to_s) # required by azure table storage
      end

      id = repo.save(attributes)
      id.should_not be_nil 

      db_instance = repo.find_by_key(id, _build_find_by_opts(attributes, false))
      db_instance.should_not be_nil
      db_instance[:email].should eq(email)
      db_instance[:age].should eq(age)
      db_instance[:amt].to_f.should eq(amt)
      db_instance[:active].should eq(active)

      db_instance
    end

    def _create_new_instance_and_delete(logical_delete=nil)
      instance = _create_new_instance_and_verify
      id = _get_db_instance_id(instance)

      opts = {}
      opts.merge!(:row_key => instance[:row_key]) if instance[:row_key]
      opts.merge!(:logical_only => logical_delete) unless logical_delete.nil?

      repo.delete(id, opts)

      if logical_delete || (logical_delete.nil? && RD3.config.enable_logical_deletes)
        repo.find_by_key(id, _build_find_by_opts(instance, true)).should be_nil
        repo.find_by_key(id, _build_find_by_opts(instance, false)).should_not be_nil
      else
        repo.find_by_key(id, _build_find_by_opts(instance, false)).should be_nil
      end
    end

    def _get_db_instance_id(instance)
      instance[:id] || instance[:partition_key]
    end

    def _build_find_by_opts(instance, logical_only)
      {:row_key => instance[:row_key], :logical_only => logical_only}
    end

  end

end
